'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var vueDemi = require('vue-demi');
var shared = require('@vueuse/shared');

function createGenericProjection(fromDomain, toDomain, projector) {
  return (input) => {
    return vueDemi.computed(() => projector(shared.resolveUnref(input), shared.resolveUnref(fromDomain), shared.resolveUnref(toDomain)));
  };
}

const defaultNumericProjector = (input, from, to) => {
  return (input - from[0]) / (from[1] - from[0]) * (to[1] - to[0]) + to[0];
};
function createProjection(fromDomain, toDomain, projector = defaultNumericProjector) {
  return createGenericProjection(fromDomain, toDomain, projector);
}

function logicAnd(...args) {
  return vueDemi.computed(() => args.every((i) => shared.resolveUnref(i)));
}

function logicNot(v) {
  return vueDemi.computed(() => !shared.resolveUnref(v));
}

function logicOr(...args) {
  return vueDemi.computed(() => args.some((i) => shared.resolveUnref(i)));
}

function useAbs(value) {
  return vueDemi.computed(() => Math.abs(shared.resolveUnref(value)));
}

function resolveUnrefArgsFlat(args) {
  return args.flatMap((i) => {
    const v = shared.resolveUnref(i);
    if (Array.isArray(v))
      return v.map((i2) => shared.resolveUnref(i2));
    return [v];
  });
}

function useAverage(...args) {
  return vueDemi.computed(() => {
    const array = resolveUnrefArgsFlat(args);
    return array.reduce((sum, v) => sum += v, 0) / array.length;
  });
}

function useCeil(value) {
  return vueDemi.computed(() => Math.ceil(shared.resolveUnref(value)));
}

function useClamp(value, min, max) {
  const _value = vueDemi.ref(value);
  return vueDemi.computed({
    get() {
      return _value.value = shared.clamp(_value.value, shared.resolveUnref(min), shared.resolveUnref(max));
    },
    set(value2) {
      _value.value = shared.clamp(value2, shared.resolveUnref(min), shared.resolveUnref(max));
    }
  });
}

function useFloor(value) {
  return vueDemi.computed(() => Math.floor(shared.resolveUnref(value)));
}

function useMath(key, ...args) {
  return shared.reactify(Math[key])(...args);
}

function useMax(...args) {
  return vueDemi.computed(() => {
    const array = resolveUnrefArgsFlat(args);
    return Math.max(...array);
  });
}

function useMin(...args) {
  return vueDemi.computed(() => {
    const array = resolveUnrefArgsFlat(args);
    return Math.min(...array);
  });
}

function useProjection(input, fromDomain, toDomain, projector) {
  return createProjection(fromDomain, toDomain, projector)(input);
}

function useRound(value) {
  return vueDemi.computed(() => Math.round(shared.resolveUnref(value)));
}

function useSum(...args) {
  return vueDemi.computed(() => resolveUnrefArgsFlat(args).reduce((sum, v) => sum += v, 0));
}

function useTrunc(value) {
  return vueDemi.computed(() => Math.trunc(shared.resolveUnref(value)));
}

exports.and = logicAnd;
exports.createGenericProjection = createGenericProjection;
exports.createProjection = createProjection;
exports.logicAnd = logicAnd;
exports.logicNot = logicNot;
exports.logicOr = logicOr;
exports.not = logicNot;
exports.or = logicOr;
exports.useAbs = useAbs;
exports.useAverage = useAverage;
exports.useCeil = useCeil;
exports.useClamp = useClamp;
exports.useFloor = useFloor;
exports.useMath = useMath;
exports.useMax = useMax;
exports.useMin = useMin;
exports.useProjection = useProjection;
exports.useRound = useRound;
exports.useSum = useSum;
exports.useTrunc = useTrunc;
