import { ComputedRef, Ref } from 'vue-demi';
import { MaybeComputedRef, MaybeRef, ArgumentsType, Reactified } from '@vueuse/shared';
import * as vue from 'vue-demi';

declare type ProjectorFunction<F, T> = (input: F, from: readonly [F, F], to: readonly [T, T]) => T;
declare type UseProjection<F, T> = (input: MaybeComputedRef<F>) => ComputedRef<T>;
declare function createGenericProjection<F = number, T = number>(fromDomain: MaybeComputedRef<readonly [F, F]>, toDomain: MaybeComputedRef<readonly [T, T]>, projector: ProjectorFunction<F, T>): UseProjection<F, T>;

declare function createProjection(fromDomain: MaybeComputedRef<readonly [number, number]>, toDomain: MaybeComputedRef<readonly [number, number]>, projector?: ProjectorFunction<number, number>): UseProjection<number, number>;

/**
 * `AND` conditions for refs.
 *
 * @see https://vueuse.org/logicAnd
 */
declare function logicAnd(...args: MaybeComputedRef<any>[]): ComputedRef<boolean>;

/**
 * `NOT` conditions for refs.
 *
 * @see https://vueuse.org/logicNot
 */
declare function logicNot(v: MaybeComputedRef<any>): ComputedRef<boolean>;

/**
 * `OR` conditions for refs.
 *
 * @see https://vueuse.org/logicOr
 */
declare function logicOr(...args: MaybeComputedRef<any>[]): ComputedRef<boolean>;

/**
 * Reactive `Math.abs`.
 *
 * @see https://vueuse.org/useAbs
 */
declare function useAbs(value: MaybeComputedRef<number>): ComputedRef<number>;

declare function useAverage(array: MaybeComputedRef<MaybeComputedRef<number>[]>): ComputedRef<number>;
declare function useAverage(...args: MaybeComputedRef<number>[]): ComputedRef<number>;

/**
 * Reactive `Math.ceil`.
 *
 * @see https://vueuse.org/useCeil
 */
declare function useCeil(value: MaybeComputedRef<number>): ComputedRef<number>;

/**
 * Reactively clamp a value between two other values.
 *
 * @see https://vueuse.org/useClamp
 * @param value number
 * @param min
 * @param max
 */
declare function useClamp(value: MaybeRef<number>, min: MaybeComputedRef<number>, max: MaybeComputedRef<number>): Ref<number>;

/**
 * Reactive `Math.floor`
 *
 * @see https://vueuse.org/useFloor
 */
declare function useFloor(value: MaybeComputedRef<number>): ComputedRef<number>;

declare type UseMathKeys = keyof {
    [K in keyof Math as Math[K] extends (...args: any) => any ? K : never]: unknown;
};
/**
 * Reactive `Math` methods.
 *
 * @see https://vueuse.org/useMath
 */
declare function useMath<K extends keyof Math>(key: K, ...args: ArgumentsType<Reactified<Math[K], true>>): ReturnType<Reactified<Math[K], true>>;

declare function useMax(array: MaybeComputedRef<MaybeComputedRef<number>[]>): ComputedRef<number>;
declare function useMax(...args: MaybeComputedRef<number>[]): ComputedRef<number>;

declare function useMin(array: MaybeComputedRef<MaybeComputedRef<number>[]>): ComputedRef<number>;
declare function useMin(...args: MaybeComputedRef<number>[]): ComputedRef<number>;

/**
 * Reactive numeric projection from one domain to another.
 *
 * @see https://vueuse.org/useProjection
 */
declare function useProjection(input: MaybeComputedRef<number>, fromDomain: MaybeComputedRef<readonly [number, number]>, toDomain: MaybeComputedRef<readonly [number, number]>, projector?: ProjectorFunction<number, number>): vue.ComputedRef<number>;

/**
 * Reactive `Math.round`.
 *
 * @see https://vueuse.org/useRound
 */
declare function useRound(value: MaybeComputedRef<number>): ComputedRef<number>;

declare function useSum(array: MaybeComputedRef<MaybeComputedRef<number>[]>): ComputedRef<number>;
declare function useSum(...args: MaybeComputedRef<number>[]): ComputedRef<number>;

/**
 * Reactive `Math.trunc`.
 *
 * @see https://vueuse.org/useTrunc
 */
declare function useTrunc(value: MaybeComputedRef<number>): ComputedRef<number>;

export { ProjectorFunction, UseMathKeys, UseProjection, logicAnd as and, createGenericProjection, createProjection, logicAnd, logicNot, logicOr, logicNot as not, logicOr as or, useAbs, useAverage, useCeil, useClamp, useFloor, useMath, useMax, useMin, useProjection, useRound, useSum, useTrunc };
